'Ascii bitmap font rendering
SuperStrict

Rem
bbdoc: Gaphics/AsciiFont
End Rem
Module ODD.AsciiFont

ModuleInfo "Version: 1.0"
ModuleInfo "Author: David Williamson"
ModuleInfo "License: Public Domain"

Import BRL.Max2D

Const ASCII_PADDED:Int=16
Const ASCII_EXTENDED:Int=32

Const ALIGN_LEFT:Int=-1
Const ALIGN_RIGHT:Int=1
Const ALIGN_TOP:Int=-1
Const ALIGN_BOTTOM:Int=1
Const ALIGN_CENTER:Int=0

Type TAsciiFont
	Field img:TImage, _width:Int, _height:Int
	Field frontPad:Int, extended:Int
	
	Rem
	bbdoc: Loads an ascii font from an image file
	returns: An ascii font object
	about: Loads a fixed width ascii ordered font for rendering.
	
	@flags are the standard #LoadImage flags with two ascii font specific additions
	[ @ASCII_PADDED | The image contains the first 32 unprintable characters
	* @ASCII_EXTENDED | The image contains the extended ascii characters(128-255)
	]
	End Rem
	Function Load:TAsciiFont( url:Object, width:Int, height:Int, flags:Int=0 )
		Local af:TAsciiFont=New TAsciiFont
		af.frontPad=flags&ASCII_PADDED
		af.extended=flags&ASCII_EXTENDED
		Local fCount:Int=96
		If af.frontPad Then fCount:+32
		If af.extended Then fCount:+128
		af.img=LoadAnimImage(url,width,height,0,fCount,flags)
		af._width=width;af._height=height
		If af.img Then Return af Else Return Null
	End Function
	
	Rem
	bbdoc: Draws text on the screen
	about: Draws text at position @x, @y. @horAlign and @vertAlign discribes how the text is aligned. The values are @ALIGN_LEFT, @ALIGN_RIGHT, @ALIGN_TOP, @ALIGN_BOTTOM, @ALIGN_CENTER
	Draw ignores the current Max2D rotation
	EndRem
	Method Draw( text:String, x:Float, y:Float, horAlign:Int=ALIGN_LEFT, vertAlign:Int=ALIGN_TOP )
		Local rot:Float=GetRotation()
		Local xScale:Float, yScale:Float
		GetScale xScale, yScale
		Local fOffset:Int=0
		If Not frontPad Then fOffset=-32
		Local a:Int
		Local align:Float=Width(text)*.5*xScale
		x:-(align+align*horAlign)
		align=_height*.5*yScale
		y:-(align+align*vertAlign)
		For Local count:Int=0 Until text.length
			a=text[count]
			If Not(extended) And a>=128 Then Continue
			DrawImage img,x+(count*_width*xScale),y,a+fOffset
		Next
		SetRotation rot
	End Method
	
	Rem
	bbdoc: Gets the total width of text
	returns: The pixel width of text
	about: if text is blank the #Width returns the width of a single character.
	EndRem
	Method Width:Int( text:String="" )
		If text Then Return _width*text.length Else Return _width
	End Method
	
	Rem
	bbdoc: Gets the height of the ascii font
	EndRem
	Method Height:Int()
		Return _height
	End Method
End Type

Rem
bbdoc: Loads an ascii font from an image file
returns: An ascii font object
about: Loads a fixed width ascii ordered font for rendering.

@flags are the standard #LoadImage flags with two ascii font specific additions
[ @ASCII_PADDED | The image contains the first 32 unprintable characters
* @ASCII_EXTENDED | The image contains the extended ascii characters(128-255)
]
End Rem
Function LoadAsciiFont:TAsciiFont( url:Object, width:Int, height:Int, flags:Int=0 )
	Return TAsciiFont.Load(url,width,height,flags)
End Function